

import org.freepascal.rtl.TObject;

import SecureBlackbox.Base.*;
import SecureBlackbox.FTPSClient.*;
import SecureBlackbox.SSLCommon.TSBCertificateValidateEvent;


public class Main {
	static TElSimpleFTPSClient Client = null;

	// command line parameters
	static final int OP_LIST	 = 1;
	static final int OP_DOWNLOAD = 2;
	static final int OP_UPLOAD	 = 3;
	
	static long parOperation = 0;
	static String parRemoteName = "/";
	static String parLocalName;
	static String parUsername;
	static String parPassword;
	static String parAddress;
	static int parPort;

	static boolean ParseCommandLine(String[] args)
	{
		int i = 0;
		String arg; 

		if (args.length == 0) 
		{
			System.out.println("usage:");
			System.out.println("FTPSClient {-download|-upload} -addr[ess] <address> [-port <port>] -user[name] <user> -pass[word] <password> -remote <remote file name> -local <local file name>");
			return false;
		}
		else
		{
			while (i < args.length)
			{
				arg = args[i];
				if (arg.compareTo("-down") == 0 || 
					arg.compareTo("-download") == 0)
				{
					parOperation = OP_DOWNLOAD;
				}
				else
					if (arg.compareTo("-up") == 0 || 
					arg.compareTo("-upload") == 0)
				{
					parOperation = OP_UPLOAD;
				}
				else
					if (arg.compareTo("-list") == 0 || 
					arg.compareTo("-ls") == 0)
				{
					parOperation = OP_LIST;
				}
				else
					if (arg.compareTo("-addr") == 0 || 
					arg.compareTo("-address") == 0)
				{
					i++;
					parAddress = args[i];
				}
				else
					if (arg.compareTo("-port") == 0)
				{
					i++;
					try
					{
						parPort = Integer.parseInt(args[i]);
					}
					catch (Exception ex)
					{
						parPort = 21;
					}
				}
				else
					if (arg.compareTo("-user") == 0 || 
					arg.compareTo("-username") == 0)
				{
					i++;
					parUsername = args[i];
				}
				else
					if (arg.compareTo("-pass") == 0 || 
					arg.compareTo("-password") == 0)
				{
					i++;
					parPassword = args[i];
				}
				else					
					if (arg.compareTo("-local") == 0)
				{
					i++;
					parLocalName = args[i];
				}
				else					
					if (arg.compareTo("-remote") == 0)
				{
					i++;
					parRemoteName = args[i];
				}
				i = i + 1;
			}
			return (parOperation != 0);
		}
	}

	static void Download()
	{
		TElFileStream stream = new TElFileStream(parLocalName, "rw", true);
		try
		{
			Client.receive(parRemoteName, stream);
		}
		finally
		{
			stream.Free();
		}
	}

	static void Upload()
	{
		TElFileStream stream = new TElFileStream(parLocalName, "rw", true);
		try
		{
			Client.send(stream, parRemoteName);
		}
		finally
		{
			stream.Free();
		}
	}

	static void List()
	{
		if (Client.getExtMLSTSupported())
			Client.mlsd();
		else
			Client.getFileList();			
	}

	static TSBFTPSTextDataEvent.Callback onTextDataLine = new TSBFTPSTextDataEvent.Callback() {
		
		public void tsbftpsTextDataEventCallback(TObject sender, byte[] textLine) {
			String response = new String(textLine);
			TSBFTPFileInfo FileInfo = new TSBFTPFileInfo();

			try
			{
				if (Client.getExtMLSTSupported())
				{
					TElSimpleFTPSClient.parseMLSDEntry(null, response, FileInfo);
				}
				else
				{
					TElSimpleFTPSClient.parseFileListEntry(null, response, FileInfo);
				}
				System.out.println(FileInfo.FileName);
			}
			catch (Exception ex)
			{
				// Couldn't parse the entry, so just write what we've received
				System.out.println(response);
			}
		}
	};
	
	static TSBCertificateValidateEvent.Callback onCertificateValidate = new TSBCertificateValidateEvent.Callback() {
		
		public void tsbCertificateValidateEventCallback(TObject sender,
				TElX509Certificate cert, TSBBoolean accept) {
			System.out.println("Server certificate received");
			accept.value = true;
		}
	};
	
	public static void main(String[] args) {

		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
		SBUtils.setLicenseKey("<put the key here>");

		if (!ParseCommandLine(args))
			return;

		Client = new TElSimpleFTPSClient();
		Client.setAddress(parAddress);
		Client.setPort(parPort);

		Client.setUsername(parUsername);
		Client.setPassword(parPassword);
		Client.setUseSSL(false);
		
		Client.setOnCertificateValidate(new TSBCertificateValidateEvent(onCertificateValidate));
		Client.setOnTextDataLine(new TSBFTPSTextDataEvent(onTextDataLine));

		try
		{
			try
			{
				Client.open();
				Client.login();
			}
			catch (Exception e)
			{
				System.out.println("Connection failed due to exception: " + e.getMessage());
				System.out.println("If you have ensured that all connection parameters are correct and you still can't connect,");
				System.out.println("please contact EldoS support as described on http://www.eldos.com/sbb/support-tech.php");
				System.out.println("Remember to provide details about the error that happened.");
				try
				{
					Client.close(true);
				}
				catch (Exception ex)
				{ 
				}
				return;
			}

			try
			{
				if (parOperation == OP_DOWNLOAD)
					Download();
				else if (parOperation == OP_UPLOAD)
					Upload();
				else if (parOperation == OP_LIST)
					List();
			}
			finally
			{
				Client.close(false);
			}
		}
		catch (Exception e)
		{
			System.out.println("Exception: " + e.getMessage());
		}
	}
}
