import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.JCheckBox;
import javax.swing.JPasswordField;

import SecureBlackbox.Base.*;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;


public class SettingsDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private final JPanel contentPanel = new JPanel();
	private JTextField tbPath;
	private JPasswordField certPasw;
	private boolean isOk;
	private JSpinner spinner;
	private JButton btnBrowsePath;
	private JCheckBox chckbxAuthBasic;
	private JCheckBox chckbxAuthDigest;
	private JCheckBox chckbxUseSsltls;
	private JTextField tbCertFile;
	private TElMemoryCertStorage m_certStorage = new TElMemoryCertStorage();

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		try {
			SettingsDialog dialog = new SettingsDialog();
			dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			dialog.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Create the dialog.
	 */
	public SettingsDialog() {
		setTitle("Server Settings");
		setBounds(100, 100, 584, 330);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(null);
		
		JLabel lblBasePath = new JLabel("Base path");
		lblBasePath.setBounds(10, 11, 92, 14);
		contentPanel.add(lblBasePath);
		
		tbPath = new JTextField();
		tbPath.setBounds(10, 30, 515, 20);
		contentPanel.add(tbPath);
		tbPath.setColumns(10);
		
		btnBrowsePath = new JButton("...");
		btnBrowsePath.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				tbPath.setText(getOpenDir());
			}
		});
		btnBrowsePath.setBounds(535, 29, 30, 23);
		contentPanel.add(btnBrowsePath);
		
		JLabel lblListenOnPort = new JLabel("Listen on port");
		lblListenOnPort.setBounds(10, 61, 92, 14);
		contentPanel.add(lblListenOnPort);
		
		spinner = new JSpinner();
		spinner.setModel(new SpinnerNumberModel(new Integer(8080), null, null, new Integer(1)));
		spinner.setBounds(94, 58, 60, 20);
		contentPanel.add(spinner);
		
		chckbxUseSsltls = new JCheckBox("Use SSL/TLS");
		chckbxUseSsltls.setBounds(187, 57, 101, 23);
		contentPanel.add(chckbxUseSsltls);
		
		
		chckbxAuthBasic = new JCheckBox("Basic authentication");
		chckbxAuthBasic.setBounds(10, 207, 151, 23);
		contentPanel.add(chckbxAuthBasic);

		
		chckbxAuthDigest = new JCheckBox("Digest authentication");
		chckbxAuthDigest.setBounds(187, 207, 151, 23);
		contentPanel.add(chckbxAuthDigest);
		
		
		JLabel lblCertificateFile = new JLabel("Certificate file");
		lblCertificateFile.setBounds(10, 90, 154, 14);
		contentPanel.add(lblCertificateFile);
		
		tbCertFile = new JTextField();
		tbCertFile.setColumns(10);
		tbCertFile.setBounds(10, 109, 515, 20);
		contentPanel.add(tbCertFile);
		
		JButton buttonBrowseCert = new JButton("...");
		buttonBrowseCert.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				tbCertFile.setText(getOpenFileName());
			}
		});
		buttonBrowseCert.setBounds(535, 108, 30, 23);
		contentPanel.add(buttonBrowseCert);
		
		JLabel lblCertificatePassword = new JLabel("Certificate password");
		lblCertificatePassword.setBounds(10, 142, 260, 14);
		contentPanel.add(lblCertificatePassword);
		
		certPasw = new JPasswordField();
		certPasw.setBounds(10, 167, 334, 20);
		contentPanel.add(certPasw);
		{
			JPanel buttonPane = new JPanel();
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				JButton okButton = new JButton("OK");
				okButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						
						if (chckbxUseSsltls.isSelected())
						{
							TElX509Certificate Cert = new TElX509Certificate();
							if (Cert.loadFromFileAuto(tbCertFile.getText(), new String(certPasw.getPassword())) != 0)
							{
								showMessageBox("Can not load certificate!");
							}
							else
							{
								m_certStorage.clear();
								m_certStorage.add(Cert, true);
							}
						}
						
						isOk = true;
						setVisible(false);
					}
				});
				okButton.setActionCommand("OK");
				buttonPane.add(okButton);
				getRootPane().setDefaultButton(okButton);
			}
			{
				JButton cancelButton = new JButton("Cancel");
				cancelButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						dispose();
					}
				});
				cancelButton.setActionCommand("Cancel");
				buttonPane.add(cancelButton);
			}
		}
	}

	protected String getOpenDir() {
		JFileChooser fc = new JFileChooser();
		fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	    int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}

	String getOpenFileName(){
		JFileChooser fc = new JFileChooser();
	    int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	} 

	protected void showMessageBox(String msg) {
		JOptionPane.showMessageDialog(null, msg, getTitle(), JOptionPane.INFORMATION_MESSAGE);		
	}

	public boolean isOK() {
		return isOk;
	}

	public int getPort() {
		return (Integer)spinner.getValue();
	}

	public boolean getUseSSL() {
		return chckbxUseSsltls.isSelected();
	}
	
	public boolean getAuthBasic() {
		return chckbxAuthBasic.isSelected();
	}

	public boolean getAuthDigest() {
		return chckbxAuthDigest.isSelected();
	}


	public TElCustomCertStorage getCertStorage() {
		return m_certStorage;
	}

	public String getBasePath() {
		return tbPath.getText();
	}
}
