import java.util.Vector;

import javax.swing.table.AbstractTableModel;

import SecureBlackbox.Office.TElOfficeCustomSignatureHandler;
import SecureBlackbox.Office.TElOfficeOpenXMLBaseSignatureHandler;
import SecureBlackbox.Office.TElOfficePackagePart;
import SecureBlackbox.Office.TSBOfficeOpenXMLPartValidationStatus;

public class PartsTableModel extends AbstractTableModel {
	private static final long serialVersionUID = 2771284071470299948L;
	public static final int URI_INDEX = 0;
	public static final int STATE_INDEX = 1;
	
	protected Vector<PartsDataRow> dataVector = new Vector<PartsDataRow>();
	
    protected String[] columnNames = {
    		"URI", 
    		"State", 
    };
    
	public String getColumnName(int column) {
        return columnNames[column];
    }
    
	public int getColumnCount() {
		return columnNames.length;
	}

	public int getRowCount() {
		return dataVector.size();
	}

    public Object getValueAt(int row, int col) {
        PartsDataRow record = (PartsDataRow)dataVector.get(row);
        switch (col) {
        	case URI_INDEX:
	        	 return record.URI;
            case STATE_INDEX:
               return record.State;
            default:
               return new Object();
        }
    }

	public void clear() {
		dataVector.removeAllElements();
	}

    public void addRow(String URI, String state, Object tag){
    	dataVector.addElement(new PartsDataRow(URI, state, tag));
    	fireTableDataChanged();
    }
    
	public void fill(TElOfficeCustomSignatureHandler _Handler, boolean isShowAllParts) {
		
        if (_Handler instanceof TElOfficeOpenXMLBaseSignatureHandler)
        {
            TElOfficeOpenXMLBaseSignatureHandler OpenXMLBaseHandler = (TElOfficeOpenXMLBaseSignatureHandler)_Handler;
            for (int i = 0; i < OpenXMLBaseHandler.getSignedPartCount(); i++)
            {
                String URI = OpenXMLBaseHandler.getSignedPart(i).getURI();
                Object tag = OpenXMLBaseHandler.getSignedPart(i);
                TSBOfficeOpenXMLPartValidationStatus status = OpenXMLBaseHandler.getSignedPart(i).getValidationStatus();
                String state = "Unknown";
                if (status == TSBOfficeOpenXMLPartValidationStatus.pvsValid) state = "Valid";
                if (status == TSBOfficeOpenXMLPartValidationStatus.pvsInvalidDigest) state = "Invalid Digest";
                if (status == TSBOfficeOpenXMLPartValidationStatus.pvsInvalidSignature) state = "Invalid Signature";
                if (status == TSBOfficeOpenXMLPartValidationStatus.pvsInvalidContentType) state = "Invalid ContentType";
                if (status == TSBOfficeOpenXMLPartValidationStatus.pvsPartIsMissing)  state = "Part Is Missing";
                
                addRow(URI, state, tag);
            }

            if (isShowAllParts)
            {
                for (int i = 0; i < OpenXMLBaseHandler.getPackage().getPartCount(); i++)
                {
                    TElOfficePackagePart Part = OpenXMLBaseHandler.getPackage().getPartByIndex(i);
                    if (OpenXMLBaseHandler.getSignedPartByURI(Part.getURI()) == null)
                    {
                        String URI = Part.getURI();
                        Object tag = Part;
                        String state = "Not signed";
                        addRow(URI, state, tag);
                    }
                }
            }
        }
     }

	public Object getTag(int selectedRow) {
		return dataVector.get(selectedRow).Tag;
	}
}