import java.awt.Font;

import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import SecureBlackbox.PGP.SBPGPUtils;
import SecureBlackbox.PGP.TElPGPKeyring;
import SecureBlackbox.PGP.TElPGPPublicKey;
import SecureBlackbox.PGP.TElPGPSignature;
import SecureBlackbox.PGP.TElPGPUserID;


public class Utils {

	
	public static void RedrawKeyring(JTree tree, TElPGPKeyring keyring, DefaultMutableTreeNode rootNode) {
		JTree tv = tree;
		int i, j, k;
		CustomTreeCellRenderer render = new CustomTreeCellRenderer();
		tree.setCellRenderer(render);
		DefaultMutableTreeNode KeyNode, UserNode, SubKeyNode, SigNode;
		Font NodeFont = tree.getFont();
		String nodeTitle = "";
		
		rootNode.removeAllChildren();
		for(i = 0; i < keyring.getPublicCount(); i++) 
		{
			nodeTitle = GetDefaultUserID(keyring.getPublicKeys(i));
			KeyNode = new DefaultMutableTreeNode(nodeTitle);
			rootNode.add(KeyNode);
				
			//KeyNode.SelectedImageIndex = KeyNode.ImageIndex;
			if (keyring.getPublicKeys(i).getSecretKey() != null) 
			{
				//KeyNode.NodeFont = new Font(this.Font, FontStyle.Bold);
			}
			if (keyring.getPublicKeys(i).getRevoked()) 
			{
				NodeFont = new Font(tv.getFont().getFontName(), Font.ITALIC, 10);
			}

			/* Creating key node */
			KeyNode.setUserObject(new NodeData(keyring.getPublicKeys(i), nodeTitle, NodeFont));

			/* Creating user nodes */
			for(j = 0; j < keyring.getPublicKeys(i).getUserIDCount(); j++) 
			{
				TElPGPUserID userID = keyring.getPublicKeys(i).getUserID(j);
				nodeTitle = userID.getName();
				UserNode = new DefaultMutableTreeNode(nodeTitle);
				UserNode.setUserObject(new NodeData(userID, nodeTitle, tree.getFont()));
				KeyNode.add(UserNode);

				/* Creating signature nodes */
				for(k = 0; k < userID.getSignatureCount(); k++) 
				{
					TElPGPSignature signature = userID.getSignature(k);
					if (signature.isUserRevocation()) 
					{
						nodeTitle = "Revocation";
						SigNode = new DefaultMutableTreeNode(nodeTitle);
						UserNode.add(SigNode);
						NodeFont = new Font(tv.getFont().getFontName(), Font.ITALIC, 10);
					} 
					else 
					{
						nodeTitle = "Signature";
						SigNode = new DefaultMutableTreeNode(nodeTitle); 
						UserNode.add(SigNode);
					}
					SigNode.setUserObject(new NodeData(signature, nodeTitle, NodeFont));
				}
			}
			for(j = 0; j < keyring.getPublicKeys(i).getUserAttrCount(); j++) 
			{
				nodeTitle = "Photo";
				UserNode = new DefaultMutableTreeNode(nodeTitle);
				KeyNode.add(UserNode);
				UserNode.setUserObject(new NodeData(keyring.getPublicKeys(i).getUserAttr(j), nodeTitle, tree.getFont()));
				/* Creating signature nodes */
				for(k = 0; k < keyring.getPublicKeys(i).getUserAttr(j).getSignatureCount(); k++) 
				{
					TElPGPSignature signature = keyring.getPublicKeys(i).getUserAttr(j).getSignature(k);
					if (signature.isUserRevocation()) 
					{
						nodeTitle = "Revocation";
						SigNode = new DefaultMutableTreeNode(nodeTitle);
						UserNode.add(SigNode);
						NodeFont = new Font(tv.getFont().getFontName(), Font.ITALIC, 10);
					} 
					else 
					{
						nodeTitle = "Signature";
						SigNode = new DefaultMutableTreeNode(nodeTitle);
						UserNode.add(SigNode);
					}
					SigNode.setUserObject(new NodeData(signature, nodeTitle, NodeFont));
				}
			}

			/* Sub keys */
			for(j = 0; j < keyring.getPublicKeys(i).getSubkeyCount(); j++) 
			{
				nodeTitle = SBPGPUtils.pkAlg2Str(keyring.getPublicKeys(i).getSubkey(j).getPublicKeyAlgorithm()) + " subkey";
				SubKeyNode = new DefaultMutableTreeNode(nodeTitle);
				KeyNode.add(SubKeyNode);
				SubKeyNode.setUserObject(new NodeData(keyring.getPublicKeys(i).getSubkey(j), nodeTitle, tree.getFont()));
				/* Creating signature nodes */
				for(k = 0; k < keyring.getPublicKeys(i).getSubkey(j).getSignatureCount(); k++) 
				{
					TElPGPSignature signature = keyring.getPublicKeys(i).getSubkey(j).getSignatures(k);
					if (signature .isSubkeyRevocation()) 
					{
						nodeTitle = "Revocation";
						SigNode = new DefaultMutableTreeNode(nodeTitle);
						SubKeyNode.add(SigNode);
						NodeFont = new Font(tv.getFont().getFontName(), Font.ITALIC, 10);
					} 
					else 
					{
						nodeTitle = "Signature";
						SigNode = new DefaultMutableTreeNode("Signature");
						SubKeyNode.add(SigNode);
					}
					SigNode.setUserObject(new NodeData(signature, nodeTitle, NodeFont));
				}
			}
		}	
		refreshTree(tree);
	}

	private static void refreshTree(JTree tree) {
		DefaultTreeModel model = ((DefaultTreeModel)tree.getModel());
		model.reload();
		expandTree(tree);
	}

	private static void expandTree(JTree tree) {	
	    int row = 0;
	    while (row < tree.getRowCount()) {
	      tree.expandRow(row);
	      row++;
	    }
	}
	
	public static String GetDefaultUserID(TElPGPPublicKey key)
	{
		String result;
		if (key.getUserIDCount() > 0) 
		{
			result = key.getUserID(0).getName();
		} 
		else 
		{
			result = "No name";
		}
		return result;
	}
}
