import java.awt.EventQueue;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;

import SecureBlackbox.LDAP.*;
import SecureBlackbox.PGP.*;
import SecureBlackbox.PGPLDAP.*;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;


public class FindKeyDialog extends JDialog {
	private static final long serialVersionUID = 1L;
	private JTextField tbAddress;
	private JTable lvFilters;
	private JComboBox<String> cbAttribute;
	private JComboBox<String> cbEquality;
	private JComboBox<String> cbValue;
	private JTree tree;
	private JTextField tbValue;
	private JTextField dpDate;
	private DefaultMutableTreeNode rootNode;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					FindKeyDialog dialog = new FindKeyDialog();
					dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
					dialog.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the dialog.
	 */
	public FindKeyDialog() {
		setTitle("Search keys");
		setBounds(100, 100, 594, 327);
		getContentPane().setLayout(null);
		
		JLabel lblSearchForKeys = new JLabel("Search for keys on");
		lblSearchForKeys.setBounds(10, 11, 125, 14);
		getContentPane().add(lblSearchForKeys);
		
		tbAddress = new JTextField();
		tbAddress.setText("keyserver.pgp.com");
		tbAddress.setBounds(136, 8, 210, 20);
		getContentPane().add(tbAddress);
		tbAddress.setColumns(10);
		
		cbAttribute = new JComboBox<String>();
		cbAttribute.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				selectedAttrChanged(e);
			}
		});
		cbAttribute.setModel(new DefaultComboBoxModel<String>(new String[] {"User ID", "Key ID", "Key Type", "Creation Date", "Expiration Date", "Key Status", "Key Size"}));
		cbAttribute.setBounds(10, 46, 125, 22);
		getContentPane().add(cbAttribute);
		
		cbEquality = new JComboBox<String>();
		cbEquality.setModel(new DefaultComboBoxModel<String>(new String[] {"User ID", "Key ID", "Key Type", "Creation Date", "Expiration Date", "Key Status", "Key Size"}));
		cbEquality.setBounds(146, 46, 155, 22);
		getContentPane().add(cbEquality);
		
		cbValue = new JComboBox<String>();
		cbValue.setBounds(311, 46, 155, 22);
		getContentPane().add(cbValue);
		
		JButton btnAdd = new JButton("Add");
		btnAdd.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				addClick();
			}
		});
		btnAdd.setBounds(488, 79, 91, 23);
		getContentPane().add(btnAdd);
		
		JButton btnRemove = new JButton("Remove");
		btnRemove.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeClick();
			}
		});
		btnRemove.setBounds(488, 113, 91, 23);
		getContentPane().add(btnRemove);
		
		JButton btnSearch = new JButton("Search");
		btnSearch.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				searchClick();
			}
		});
		btnSearch.setBounds(488, 147, 91, 23);
		getContentPane().add(btnSearch);
		
		JScrollPane scrollPane = new JScrollPane();
		scrollPane.setBounds(10, 79, 457, 101);
		getContentPane().add(scrollPane);
		
		lvFilters = new JTable(new FiltersTableModel());
		lvFilters.setFillsViewportHeight(true);
		scrollPane.setViewportView(lvFilters);
		
		JScrollPane scrollPane_1 = new JScrollPane();
		scrollPane_1.setBounds(10, 190, 572, 101);
		getContentPane().add(scrollPane_1);
		
		rootNode = new DefaultMutableTreeNode();
		tree = new JTree(rootNode);
		tree.setRootVisible(false);
		tree.setShowsRootHandles(true); 				
		scrollPane_1.setViewportView(tree);
		
		tbValue = new JTextField();
		tbValue.setVisible(false);
		tbValue.setBounds(311, 47, 155, 20);
		getContentPane().add(tbValue);
		tbValue.setColumns(10);
		
		dpDate = new JTextField(new Date().toString());
		dpDate.setVisible(false);
		dpDate.setColumns(10);
		dpDate.setBounds(311, 47, 155, 20);
		getContentPane().add(dpDate);

	}

	protected void selectedAttrChanged(ItemEvent e) {
		cbValue.setSelectedItem("");

		String attr = (String)cbAttribute.getSelectedItem();
		if (attr.compareTo("User ID") == 0)
		{
			tbValue.setVisible(true);
			cbValue.setVisible(false);
			dpDate.setVisible(false);

			cbEquality.removeAll();
			cbEquality.addItem("is equal");
			cbEquality.addItem("is not equal");
			cbEquality.addItem("contains");
			cbEquality.addItem("does not contain");
		}
		else if (attr.compareTo("Key ID") == 0)
		{
			tbValue.setVisible(true);
			cbValue.setVisible(false);
			dpDate.setVisible(false);

			cbEquality.removeAll();
			cbEquality.addItem("is equal");
			cbEquality.addItem("is not equal");
		}
		else if (attr.compareTo("Key Type") == 0)
		{
			tbValue.setVisible(false);
			cbValue.setVisible(true);
			dpDate.setVisible(false);

			cbEquality.removeAll();
			cbEquality.addItem("is equal");

			cbValue.removeAll();
			cbValue.addItem("DSS/DH");
			cbValue.addItem("RSA");
		}
		else if (attr.compareTo("Creation Date") == 0)
		{
			tbValue.setVisible(false);
			cbValue.setVisible(false);
			dpDate.setVisible(true);

			cbEquality.removeAll();
			cbEquality.addItem("less or equal");
			cbEquality.addItem("greater or equal");
		}
		else if (attr.compareTo("Expiration Date") == 0)
		{
			tbValue.setVisible(false);
			cbValue.setVisible(false);
			dpDate.setVisible(true);

			cbEquality.removeAll();
			cbEquality.addItem("less or equal");
			cbEquality.addItem("greater or equal");
		}
		else if (attr.compareTo("Key Status") == 0)
		{
			tbValue.setVisible(false);
			cbValue.setVisible(true);
			dpDate.setVisible(false);

			cbEquality.removeAll();
			cbEquality.addItem("is equal");
			cbEquality.addItem("is not equal");

			cbValue.removeAll();
			cbValue.addItem("revoked");
			cbValue.addItem("disabled");
		}
		else if (attr.compareTo("Key Size") == 0)
		{
			tbValue.setVisible(false);
			cbValue.setVisible(true);
			dpDate.setVisible(false);

			cbEquality.removeAll();
			cbEquality.addItem("is equal");
			cbEquality.addItem("less or equal");
			cbEquality.addItem("greater or equal");

			cbValue.removeAll();
			cbValue.addItem("768");
			cbValue.addItem("1024");
			cbValue.addItem("2048");
			cbValue.addItem("4096");
		}
	}

	protected void searchClick() {
		try
		{
			TElLDAPSKeyserverClient cli = new TElLDAPSKeyserverClient();
			TElLDAPSKeyserverSearchFilter filter = new TElLDAPSKeyserverSearchFilter();
			TSBLDAPSEquality equality = TSBLDAPSEquality.leEqual;
			TSBLDAPSKeyserverAttribute attr = TSBLDAPSKeyserverAttribute.lkaUserID;
			TElPGPKeyring res = null;

			tree.removeAll();

			cli.setAddress(tbAddress.getText());
			cli.setPort(389);

			cli.setIncludeDisabledKeys(true);
			cli.setIncludeRevokedKeys(true);

			for (int i = 0; i < lvFilters.getRowCount(); i++)
			{
				FiltersTableModel model = (FiltersTableModel)lvFilters.getModel();
				
				FiltersDataRow row = model.getItem(i);
				String eq = row.Equality;
				
				if (eq.compareTo("is equal") == 0)
					equality = TSBLDAPSEquality.leEqual;
				else if (eq.compareTo("is not equal") == 0)
					equality = TSBLDAPSEquality.leNotEqual;
				else if (eq.compareTo("contains") == 0)
					equality = TSBLDAPSEquality.leContains;
				else if (eq.compareTo("does not contain") == 0)
					equality = TSBLDAPSEquality.leDoesNotContain;
				else if (eq.compareTo("less or equal") == 0)
					equality = TSBLDAPSEquality.leLessOrEqual;
				else if (eq.compareTo("greater or equal") == 0)
					equality = TSBLDAPSEquality.leGreaterOrEqual;
                 
				String sattr = row.Attribute;
				String value = row.Value;
				
				if (sattr.compareTo("User ID") == 0)
				{
					attr = TSBLDAPSKeyserverAttribute.lkaUserID;
				}
				else if (sattr.compareTo("Key ID") == 0)
				{
					attr = TSBLDAPSKeyserverAttribute.lkaKeyID;
				}
				else if (sattr.compareTo("Key Type") == 0)
				{
					attr = TSBLDAPSKeyserverAttribute.lkaKeyType;
				}
				else if (sattr.compareTo("Creation Date") == 0)
				{
					attr = TSBLDAPSKeyserverAttribute.lkaKeyCreateTime;
				}
				else if (sattr.compareTo("Expiration Date") == 0)
				{
					attr = TSBLDAPSKeyserverAttribute.lkaKeyExpireTime;
				}
				else if ((sattr.compareTo("Key Status") == 0) && (value.compareTo("revoked") == 0))
				{
					attr = TSBLDAPSKeyserverAttribute.lkaRevoked;
					value = "1";
				}
				else if ((sattr.compareTo("Key Status") == 0) && (value.compareTo("disabled") == 0))
				{
					attr = TSBLDAPSKeyserverAttribute.lkaDisabled;
					value = "1";
				}
				else if (sattr.compareTo("Key Size") == 0)
				{
					attr = TSBLDAPSKeyserverAttribute.lkaKeySize;
				}

				filter.addCondition(TSBLDAPSLogicalOperator.loAnd, attr, equality, value);
			}

			cli.bind();
			res = cli.search(filter);
			cli.unbind();

			if (res.getPublicCount() > 0)
				Utils.RedrawKeyring(tree, res, rootNode);
			else
				showMessage("No keys found", "Result");
		}
		catch (Exception ex)
		{
			showErrorMessage(ex.getMessage(), "Error");
		}
	}

	private void showMessage(String msg, String cap) {
		JOptionPane.showMessageDialog(null, msg, cap, JOptionPane.INFORMATION_MESSAGE);				
	}

	private void showErrorMessage(String msg, String cap) {
		JOptionPane.showMessageDialog(null, msg, cap, JOptionPane.ERROR_MESSAGE);				
	}
	
	protected void removeClick() {
		int [] inds = lvFilters.getSelectedRows();
		for (int i = 0; i < inds.length; i++)
			lvFilters.remove(i);
	}

	protected void addClick() {
		String Value = "";

		if (cbValue.isVisible())
		{
			Value = (String)cbValue.getSelectedItem();
		}
		else if (tbValue.isVisible())
		{
			Value = tbValue.getText();
		}
		else if (dpDate.isVisible())
		{
            SimpleDateFormat format = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss z");
                        
            Date date = new Date();
			try {
				date = format.parse(dpDate.getText());
			} catch (ParseException e) {
				e.printStackTrace();
			}
			
            Calendar gcalendar = new GregorianCalendar();
            gcalendar.setTime(date);
			
            Value = gcalendar.get(Calendar.YEAR) + gcalendar.get(Calendar.MONTH) + gcalendar.get(Calendar.DAY_OF_MONTH) + "";
		}

		FiltersTableModel model = (FiltersTableModel)lvFilters.getModel();
		String attr = (String)cbAttribute.getSelectedItem();
		String equ = (String)cbEquality.getSelectedItem();
		model.addRow(attr, equ, Value);
	}
}
