import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Date;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import javax.swing.JLabel;
import javax.swing.JComboBox;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JCheckBox;

import org.freepascal.rtl.TObject;

import SecureBlackbox.Base.*;
import SecureBlackbox.HTTPClient.*;
import SecureBlackbox.PDF.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

public class SigOpDialog extends JDialog {

	private static final long serialVersionUID = 1L;
	private final JPanel contentPanel = new JPanel();
	private JTextField tbAuthor;
	private JTextField tbURL;
	private boolean isOk;
	private JCheckBox chckbxRequestATimestamp;
	private JComboBox<String> cmbSigntype;
	private JComboBox<String> cmbFont;
	private JComboBox<String> cmbReason;

    private SelCertDialog frmSelectCertificate;
    private TElPDFPublicKeySecurityHandler PublicKeyHandler;
    private TElX509Certificate FCert;
    private TElMemoryCertStorage CertStorage;
    private TElHTTPSClient HTTPClient;
    private TElHTTPTSPClient TSPClient;


	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		try {
			SigOpDialog dialog = new SigOpDialog();
			dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			dialog.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Create the dialog.
	 */
	public SigOpDialog() {
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				if (PublicKeyHandler != null)
					PublicKeyHandler.Destroy();
				if (FCert != null)
					FCert.Destroy();
				if (CertStorage != null)
					CertStorage.Destroy();
				if (HTTPClient != null)
					HTTPClient.Destroy();
				if (TSPClient != null)
					TSPClient.Destroy();
			}
		});	
		
		setTitle("Signing options");
		setBounds(100, 100, 392, 384);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(null);
		{
			JPanel panel = new JPanel();
			panel.setBorder(new TitledBorder(null, "", TitledBorder.LEADING, TitledBorder.TOP, null, null));
			panel.setBounds(10, 11, 365, 302);
			contentPanel.add(panel);
			panel.setLayout(null);
			{
				JButton btnSelectCertificate = new JButton("Select certificate");
				btnSelectCertificate.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						frmSelectCertificate.setModal(true);
						frmSelectCertificate.setVisible(true);
			            if (frmSelectCertificate.isOK())
			            {
			                if (FCert != null)
			                    FCert.Destroy();
			                try {
								FCert = frmSelectCertificate.getCert();
							} catch (Exception ex) {
								showErrorMessage(ex);
								ex.printStackTrace();
							}
			            }
					}
				});
				btnSelectCertificate.setBounds(10, 11, 190, 23);
				panel.add(btnSelectCertificate);
			}
			{
				JLabel lblSignatureType = new JLabel("Signature type");
				lblSignatureType.setBounds(10, 45, 110, 14);
				panel.add(lblSignatureType);
			}
			{
				cmbSigntype = new JComboBox<String>();
				cmbSigntype.setModel(new DefaultComboBoxModel<String>(new String[] {"Invisible document signature", "Visible document signature", "Certification (MDP) signature"}));
				cmbSigntype.setBounds(10, 63, 335, 20);
				panel.add(cmbSigntype);
			}
			{
				JLabel lblWidgetFontAnd = new JLabel("Widget font and encoding");
				lblWidgetFontAnd.setBounds(10, 93, 249, 14);
				panel.add(lblWidgetFontAnd);
			}
			{
				cmbFont = new JComboBox<String>();
				cmbFont.setModel(new DefaultComboBoxModel<String>(new String[] {"Helvetica", "Helvetica, Win-1252", "Helvetica, Auto Encoding", "Helvetica-Bold", "Helvetica-Oblique", "Helvetica-BoldOblique", "Times-Roman", "Times-Bold", "Times-Italic", "Times-BoldItalic", "Courier", "Courier-Bold", "Courier-Oblique", "Courier-BoldOblique", "Symbol", "ZapfDingbats", "Free Serif, Unicode"}));
				cmbFont.setBounds(10, 111, 335, 20);
				panel.add(cmbFont);
			}
			{
				JLabel lblAuthorsName = new JLabel("Author's name");
				lblAuthorsName.setBounds(10, 140, 150, 14);
				panel.add(lblAuthorsName);
			}
			{
				tbAuthor = new JTextField();
				tbAuthor.setBounds(10, 159, 335, 20);
				panel.add(tbAuthor);
				tbAuthor.setColumns(10);
			}
			{
				JLabel lblReasonForSigning = new JLabel("Reason for signing");
				lblReasonForSigning.setBounds(10, 187, 174, 14);
				panel.add(lblReasonForSigning);
			}
			{
				cmbReason = new JComboBox<String>();
				cmbReason.setModel(new DefaultComboBoxModel<String>(new String[] {"<none>", "I am the author of this document", "I agree to the terms defined by placement of my signature on this document", "I have reviewed this document", "I attest to the accuracy and integrity of this document", "I am approving this document"}));
				cmbReason.setBounds(10, 207, 335, 20);
				panel.add(cmbReason);
			}
			{
				chckbxRequestATimestamp = new JCheckBox("Request a timestamp from server");
				chckbxRequestATimestamp.setBounds(10, 234, 302, 23);
				panel.add(chckbxRequestATimestamp);
			}
			{
				tbURL = new JTextField();
				tbURL.setText("http://");
				tbURL.setBounds(28, 264, 317, 20);
				panel.add(tbURL);
				tbURL.setColumns(10);
			}
		}
		{
			JPanel buttonPane = new JPanel();
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				JButton okButton = new JButton("OK");
				okButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						isOk = true;
						setVisible(false);
					}
				});
				okButton.setActionCommand("OK");
				buttonPane.add(okButton);
				getRootPane().setDefaultButton(okButton);
			}
			{
				JButton cancelButton = new JButton("Cancel");
				cancelButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						dispose();
					}
				});
				cancelButton.setActionCommand("Cancel");
				buttonPane.add(cancelButton);
			}
		}
		
		init();
	}

    private void init() {
        frmSelectCertificate = new SelCertDialog();
        PublicKeyHandler = new TElPDFPublicKeySecurityHandler();
        CertStorage = new TElMemoryCertStorage();
        HTTPClient = new TElHTTPSClient();
        TSPClient = new TElHTTPTSPClient();
	}

	void AddTrueTypeFont(String FontName, TElPDFSignatureWidgetProps Wid)
    {
        TElPDFCompositeFont Font0 = new TElPDFCompositeFont();
        TElPDFCIDFont CIDFont = new TElPDFCIDFont();
        TElPDFCIDSystemInfo SystemInfo = new TElPDFCIDSystemInfo();
        TElPDFCIDFontDescriptor FontDescriptor = new TElPDFCIDFontDescriptor();

		String AppPath = new java.io.File("").getAbsolutePath();;
        String FontsFolder = AppPath + "\\..\\Fonts\\";

        // Reading ttf file
        byte[] Buf;
        TElFileStream Stream = new TElFileStream(FontsFolder + FontName + ".ttf", "r", true);
        try
        {
            Buf = new byte[(int) Stream.getLength()];
            Stream.read(Buf, 0, Buf.length);
        }
        finally
        {
            Stream.Destroy();
        }
        FontDescriptor.setFontFile2(Buf);

        // Free UCS Outline Fonts: http://savannah.gnu.org/projects/freefont
        // To embed .TTF files, you need to extract the font metrics
        if (new File(FontsFolder + FontName + ".ufm").exists())
        {
            // Extracting font metrics using TTF2UFM utility
            // ttf2ufm.exe -a -F FreeSerif.ttf
            // the following code parse .ufm file

            byte[] CIDToGIDMap = new byte[256 * 256 * 2];
            FontDescriptor.setFlags(32);
            try
            {
				FileInputStream fstream = new FileInputStream(FontsFolder + FontName + ".ufm");
				DataInputStream in = new DataInputStream(fstream);
				BufferedReader sr = new BufferedReader(new InputStreamReader(in));            
	            while (true) //!sr.EndOfStream
	            {
	                String s = sr.readLine();
	                if (s == null)
	                    break;
	                s = s.trim();
	                if (s == "")
	                    continue;
	
	                String[] t = s.split(" ");
	                if (t.length < 2)
	                    continue;
	
	                if (t[0] == "U")
	                {
	                    if (t.length < 11)
	                        continue;
	
	                    int CID = Integer.parseInt(t[1]);
	                    int Width = Integer.parseInt(t[4]);
	                    if (CID >= 0)
	                    {
	                        int GID = Integer.parseInt(t[10]);
	                        if ((CID >= 0) && (CID < 0xFFFF) && (GID > 0))
	                        {
	                            CIDToGIDMap[CID * 2] = (byte)(GID >> 8);
	                            CIDToGIDMap[CID * 2 + 1] = (byte)(GID & 0xFF);
	                            CIDFont.getW().add(CID, Width);
	                        }
	
	                        if ((t.length > 13) && (CID == (int)'X'))
	                            FontDescriptor.setXHeight(Integer.parseInt(t[13]));
	                    }
	                    if ((t[7] == ".notdef") && (FontDescriptor.getMissingWidth() == 0))
	                        FontDescriptor.setMissingWidth(Width);
	                }
	                else if (t[0] == "FontName")
	                    FontDescriptor.setFontName(t[1]);
	                else if (t[0] == "Weight")
	                {
	                    if (FontDescriptor.getStemV() == 0)
	                    {
	                        s = t[1].toLowerCase();
	                        if ((s == "bold") || (s == "black"))
	                            FontDescriptor.setStemV(120);
	                        else
	                            FontDescriptor.setStemV(70);
	                    }
	                }
	                else if (t[0] == "ItalicAngle")
	                {
	                    FontDescriptor.setItalicAngle((int)Double.parseDouble(t[1]));
	                    if (FontDescriptor.getItalicAngle() != 0)
	                        FontDescriptor.setFlags(FontDescriptor.getFlags() | 64);
	                }
	                else if (t[0] == "Ascender")
	                    FontDescriptor.setAscent(Integer.parseInt(t[1]));
	                else if (t[0] == "Descender")
	                    FontDescriptor.setDescent(Integer.parseInt(t[1]));
	                // else if (t[0] == "UnderlineThickness")
	                // else if (t[0] == "UnderlinePosition")
	                else if (t[0] == "IsFixedPitch")
	                {
	                    if (t[1] == "true")
	                        FontDescriptor.setFlags(FontDescriptor.getFlags() | 1);
	                }
	                else if (t[0] == "FontBBox")
	                {
	                    if (t.length < 5)
	                        continue;
	                    FontDescriptor.setFontBBoxX1(Integer.parseInt(t[1]));
	                    FontDescriptor.setFontBBoxY1(Integer.parseInt(t[2]));
	                    FontDescriptor.setFontBBoxX2(Integer.parseInt(t[3]));
	                    FontDescriptor.setFontBBoxY2(Integer.parseInt(t[4]));
	                }
	                else if (t[0] == "CapHeight")
	                    FontDescriptor.setCapHeight(Integer.parseInt(t[1]));
	                else if (t[0] == "StdVW")
	                    FontDescriptor.setStemV(Integer.parseInt(t[1]));
	            }
	            sr.close();
	            in.close();
	            fstream.close();
            } catch(Exception ex){
            	showErrorMessage(ex);
            }
            if (FontDescriptor.getMissingWidth() == 0)
                FontDescriptor.setMissingWidth(600);
            CIDFont.setCIDToGIDMapData(CIDToGIDMap);
        }
        else if (FontName == "FreeSerif")
        {
            // simply set font metrics data
            FontDescriptor.setFontName(FontName);
            FontDescriptor.setAscent(1166);
            FontDescriptor.setDescent(-446);
            FontDescriptor.setCapHeight(1166);
            FontDescriptor.setFlags(32);
            FontDescriptor.setFontBBoxX1(-672);
            FontDescriptor.setFontBBoxY1(-446);
            FontDescriptor.setFontBBoxX2(1588);
            FontDescriptor.setFontBBoxY2(1166);
            FontDescriptor.setItalicAngle(0);
            FontDescriptor.setStemV(70);
            FontDescriptor.setMissingWidth(700);

            CIDFont.setDW(700);
            try
            {
				FileInputStream fstream = new FileInputStream(FontsFolder + FontName + ".w");
				DataInputStream in = new DataInputStream(fstream);
				BufferedReader sr = new BufferedReader(new InputStreamReader(in));                  
	            while (true) //!sr.EndOfStream
	            {
	                String s = sr.readLine();
	                if (s == null)
	                    break;
	                s = s.trim();
	                if (s == "")
	                    continue;
	
	                int i = s.indexOf(' ');
	                if (i >= 0)
	                {
	                    int CIDFirst = Integer.parseInt(s.substring(0, i));
	                    s = s.substring(i + 1).trim();
	                    i = s.indexOf(' ');
	                    if (i >= 0)
	                    {
	                        int CIDLast = Integer.parseInt(s.substring(0, i));
	                        int Width = Integer.parseInt(s.substring(i + 1));
	                        CIDFont.getW().addRange(CIDFirst, CIDLast, Width);
	                    }
	                    else
	                    {
	                        int Width = Integer.parseInt(s);
	                        CIDFont.getW().add(CIDFirst, Width);
	                    }
	                }
	            }
	            sr.close();
	            in.close();
	            fstream.close();
            } catch(Exception ex){
            	showErrorMessage(ex);
            }

            Stream = new TElFileStream(FontsFolder + FontName + ".ctg", "r", true);
            Buf = new byte[(int) Stream.getLength()];
            Stream.read(Buf, 0, Buf.length);
            Stream.Destroy();
            CIDFont.setCIDToGIDMapData(Buf);
        }

        Font0.setBaseFont(FontDescriptor.getFontName());
        Font0.setEncoding("Identity-H");
        Font0.setResourceName("T1_0"); // the name of font resource used by default signature widget
        Font0.setDescendantFonts(CIDFont);
        SystemInfo.setRegistry("Adobe");
        SystemInfo.setOrdering("UCS");
        CIDFont.setSubtype("CIDFontType2");
        CIDFont.setBaseFont(FontDescriptor.getFontName());
        CIDFont.setCIDSystemInfo(SystemInfo);
        CIDFont.setFontDescriptor(FontDescriptor);

        Wid.addFont(Font0);
        Wid.addFont(CIDFont);
        Wid.addFontObject(SystemInfo);
        Wid.addFontObject(FontDescriptor);
    }

    private void showErrorMessage(Exception ex) {
    	JOptionPane.showMessageDialog(null, ex.getMessage(), getTitle(), JOptionPane.ERROR_MESSAGE);
	}

	private String[] _GlyphNames;

    private String LookupGlyphName(Object sender, int UCS)
    {
        // Full list: http://partners.adobe.com/public/developer/en/opentype/glyphlist.txt

        if (_GlyphNames == null)
        {
            _GlyphNames = new String[0x10000];
            String Data = getURLData("http://partners.adobe.com/public/developer/en/opentype/glyphlist.txt");

            String[] Lines = Data.split("\n" );
            for (int i = 0; i < Lines.length; i++)
                if ((Lines[i].length() > 0) && (Lines[i].charAt(0) != '#'))
                {
                    String[] ss = Lines[i].split(";");
                    if (ss.length != 2)
                        continue;

                    String GlyphName = ss[0]; // glyph name
                    ss = ss[1].split(""); // codes
                    if (ss.length != 1)
                        continue; // skipping surrogate pairs

                    int code = (int)Integer.parseInt(ss[0]);
                    _GlyphNames[code] = GlyphName;
                }
        }

        if (UCS < _GlyphNames.length)
            return _GlyphNames[UCS];
        else
            return "";
    }

    private String getURLData(String url) {
    	String result = "";
		
    	try {
		
			URL u = new URL(url);
		
	        InputStream is = u.openStream();
	        
			BufferedReader sr = new BufferedReader(new InputStreamReader(is));            
		        
	        String s = "";
	        while ((s = sr.readLine()) != null) {
	            result += s;
	        }
	        
	        sr.close();
	        is.close();
		
		} catch (Exception e) {
			e.printStackTrace();
			showErrorMessage(e);
		}
        
    	return result;
	}

	public void AddSignature(TElPDFDocument Document) throws Exception
    {
        if (FCert == null)
            throw new Exception("Failed to load encryption certificate");

        // adding the signature and setting up property values
        int index = Document.addSignature();
        TElPDFSignature Sig = Document.getSignatureEntry(index);
        Sig.setAuthorName(tbAuthor.getText());
        Sig.setSigningTime(SBUtils.utcNow());
        if (((String)cmbReason.getSelectedItem()).compareTo("<none>") != 0)
            Sig.setReason((String)cmbReason.getSelectedItem());
        else
            Sig.setReason("");

        // configuring signature type
        switch (cmbSigntype.getSelectedIndex())
        {
            case 0:
                Sig.setInvisible(true);
                break;
            case 1:
                Sig.setInvisible(false);
                break;
            case 2:
                Sig.setInvisible(false);
                Sig.setSignatureType(SBPDF.stMDP);
                break;
        }

        if (!Sig.getInvisible())
        {
        	String font = (String)cmbFont.getSelectedItem();
            if (font.compareTo("Free Serif, Unicode") == 0)
                AddTrueTypeFont("FreeSerif", Sig.getWidgetProps());
            else
                if ((font.compareTo("") == 0) || (font.compareTo("Helvetica") == 0))
                    ; // Helvetica font is default
                else if (font.compareTo("Helvetica, Win-1252") == 0)
                {
                    // Adding Type 1 font
                    TElPDFSimpleFont SimpleFont = new TElPDFSimpleFont();
                    SimpleFont.setBaseFont("Helvetica");
                    Sig.getWidgetProps().addFont(SimpleFont);

                    // "A fonts encoding is the association between character codes and glyph descriptions."
                    // "Some character sets consist of more than 256 characters, including ligatures, accented characters,"
                    // "and other symbols required for high-quality typography or non-Latin writing systems."
                    // "Different encodings may select different subsets of the same character set."
                    // Adding encoding object
                    SimpleFont.setEncodingObject(new TElPDFEncoding());
                    Sig.getWidgetProps().addFontObject(SimpleFont.getEncodingObject());
                    // You can use one of predefined pdf encodings: StandardEncoding, MacRomanEncoding, WinAnsiEncoding or nothing
                    SimpleFont.getEncodingObject().setBaseEncoding("WinAnsiEncoding");
                    // Use Differences property to remap a code to the name of any glyph description that exists in the font
                    // Uncomment following line to change a Euro character (0x80 in Win-1252) to Pound Sign (Sterling)
                    // SimpleFont.EncodingObject.set_Differences(128, "sterling");

                    // for fun adding Euro character (U+20AC) to Signer Caption
                    Sig.getWidgetProps().setSignerCaption("Signer (\u20AC)");

                    // Define callback routine that will convert UTF16 Strings to our encoding (Win-1252)
                    Sig.getWidgetProps().setOnConvertStringToAnsi(new TSBPDFConvertStringToAnsiEvent(UTF16ToWin1252));
                }
                else if (font.compareTo("Helvetica, Auto Encoding") == 0)
                {
                    // This is a third method how to create signature widget with non-ASCII characters
                    // It uses Type1 fonts, and the Differences table for EncodingObject is generated internally
                    // However we need to know the correct character names in font
                    Sig.getWidgetProps().setAutoAdjustEncoding(true);
                    Sig.getWidgetProps().setOnLookupGlyphName(new TSBPDFLookupGlyphNameEvent(LookupGlyphName));

                    // for test adding Euro character (U+20AC) to the Signer Caption
                    Sig.getWidgetProps().setSignerCaption("Signer (\u20AC)");
                }
                else
                {
                    // Adding Type 1 font
                    TElPDFSimpleFont SimpleFont = new TElPDFSimpleFont();
                    SimpleFont.setBaseFont(font);
                    Sig.getWidgetProps().addFont(SimpleFont);
                }
        }

        // adding certificate to certificate storage
        CertStorage.clear();
        PublicKeyHandler.setSignatureType(TSBPDFPublicKeySignatureType.pstPKCS7SHA1);
        CertStorage.add(FCert, true);
        PublicKeyHandler.setCertStorage(CertStorage);
        PublicKeyHandler.setCustomName("Adobe.PPKMS");

        // configuring time stamping settings
        if (chckbxRequestATimestamp.isSelected())
        {
            TSPClient.setHTTPClient(HTTPClient);
            TSPClient.setHashAlgorithm(SBConstants.SB_ALGORITHM_DGST_SHA1);
            TSPClient.setURL(tbURL.getText());
            PublicKeyHandler.setTSPClient(TSPClient);
        }
        
        Sig.setHandler(PublicKeyHandler);
    }

	TSBPDFConvertStringToAnsiEvent.Callback UTF16ToWin1252 = new TSBPDFConvertStringToAnsiEvent.Callback() {
		
		public byte[] tsbpdfConvertStringToAnsiEventCallback(TObject sender,
				String s) {
			return s.getBytes();
		}
	};
	
	TSBPDFLookupGlyphNameEvent.Callback LookupGlyphName = new TSBPDFLookupGlyphNameEvent.Callback() {
		
		public String tsbpdfLookupGlyphNameEventCallback(TObject sender, int ucs) {
			return LookupGlyphName(sender, ucs);
		}
	};
	
	public Object getCert() {
		return FCert;
	}

	public int getSignatureType() {
		return cmbSigntype.getSelectedIndex();
	}

	public String getAuthor() {
		return tbAuthor.getText();
	}

	public String getReason() {
		return (String)cmbReason.getSelectedItem();
	}

	public boolean getTimestamp() {
		return chckbxRequestATimestamp.isSelected();
	}

	public boolean isOK() {
		return isOk;
	}
}
