

import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.border.EmptyBorder;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.JLabel;
import javax.swing.JTextField;

import SecureBlackbox.Base.*;
import SecureBlackbox.HTTPClient.SBHTTPCRL;
import SecureBlackbox.HTTPClient.SBHTTPOCSPClient;
import SecureBlackbox.PDF.*;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.UUID;


public class Main extends JDialog {

	private static final long serialVersionUID = 1L;
	private final JPanel contentPanel = new JPanel();
	private JTextField tbSource;
	private TElPDFDocument Document;
	private TElMemoryCertStorage CertStorage;
	private TElX509Certificate Cert;

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		try {
			Main dialog = new Main();
			dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
			dialog.setVisible(true);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Create the dialog.
	 */
	public Main() {
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				Cert.Destroy();
				CertStorage.Destroy();
				Document.Destroy();
				System.exit(0);
			}
		});			
		
		setTitle("Tiny PDF Processor");
		setBounds(100, 100, 457, 132);
		getContentPane().setLayout(new BorderLayout());
		contentPanel.setBorder(new EmptyBorder(5, 5, 5, 5));
		getContentPane().add(contentPanel, BorderLayout.CENTER);
		contentPanel.setLayout(null);
		
		JLabel lblPlease = new JLabel("Please select the PDF document");
		lblPlease.setBounds(10, 11, 382, 14);
		contentPanel.add(lblPlease);
		
		tbSource = new JTextField();
		tbSource.setBounds(10, 36, 332, 20);
		contentPanel.add(tbSource);
		tbSource.setColumns(10);
		
		JButton btnBrowse = new JButton("Browse");
		btnBrowse.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				tbSource.setText(getPDFFileName());
			}
		});
		btnBrowse.setBounds(353, 35, 91, 23);
		contentPanel.add(btnBrowse);
		{
			JPanel buttonPane = new JPanel();
			buttonPane.setLayout(new FlowLayout(FlowLayout.RIGHT));
			getContentPane().add(buttonPane, BorderLayout.SOUTH);
			{
				JButton okButton = new JButton("OK");
				okButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						okClick();
					}
				});
				okButton.setActionCommand("OK");
				buttonPane.add(okButton);
				getRootPane().setDefaultButton(okButton);
			}
			{
				JButton cancelButton = new JButton("Cancel");
				cancelButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent e) {
						dispose();
					}
				});
				cancelButton.setActionCommand("Cancel");
				buttonPane.add(cancelButton);
			}
		}
		
		init();
	}

	private void init() {
        #error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
        SBUtils.setLicenseKey("<put the key here>");

		// The following lines are required for HTTP retrieval of CRLs and OCSP in TElX509CertificateValidator to work
		SBHTTPCRL.registerHTTPCRLRetrieverFactory();
		SBHTTPOCSPClient.registerHTTPOCSPClientFactory();

		// the following two functions *must* be called before PDF functionality is used
		SBPDF.initialize();
		SBPDFSecurity.initialize();

		// creating objects
		Document = new TElPDFDocument();
		Document.setOwnActivatedSecurityHandlers(true);
		CertStorage = new TElMemoryCertStorage();
		Cert = new TElX509Certificate();
	}

	private String getTempFileName() {
		return System.getProperty("java.io.tmpdir") + "/" + UUID.randomUUID().toString();
	}

	void showMessage(String caption, String msg) {
		JOptionPane.showMessageDialog(null, msg, caption, JOptionPane.INFORMATION_MESSAGE);
	}	
	
	private void copyFile(String src, String dest, boolean overwrite) throws IOException {
        RandomAccessFile inputFile = new RandomAccessFile(src, "r");
        RandomAccessFile outputFile = new RandomAccessFile(dest, "rw");
        byte[] buf = new byte[1024];
        int r;
	    
	    if (overwrite)
         outputFile.setLength(0);
	    
        do {
         r = inputFile.read(buf);
         if (r > 0)
          outputFile.write(buf, 0, r);
        } while (r > 0);

        inputFile.close();
        outputFile.close();
	}
	
	protected void okClick() {
		String TempPath = getTempFileName();
		boolean DocumentChanged = false;
		try 
		{
			// creating a temporary file copy
			copyFile(tbSource.getText(), TempPath, true);
			// opening the temporary file
			TElFileStream F = new TElFileStream(TempPath, "rw", true);
			try 
			{
				Document.open(F);
				try 
				{
					// checking if the document is secured
					if ((!Document.getEncrypted()) && (!Document.getSigned())) 
					{
                        showMessage("Document is neither encrypted nor signed", "Information");
						return;
					}
                    // checking if the document is encrypted and trying to decrypt it
					if (Document.getEncrypted()) 
					{
						if (DisplayEncryptionInfo())
						{
							// decrypting the document
							if (Document.getEncryptionHandler() instanceof TElPDFPasswordSecurityHandler) 
							{
                                String pass = requestPassword("Please supply a password to decrypt the document:");
								// trying the supplied password
								boolean PasswordValid = false;
								TElPDFPasswordSecurityHandler PasswordHandler = 
									(TElPDFPasswordSecurityHandler)(Document.getEncryptionHandler());
								// Check for owner password
								PasswordHandler.setOwnerPassword(pass);
								if (PasswordHandler.isOwnerPasswordValid()) 
									PasswordValid = true;
								else 
								{
									// Check for user password
									PasswordHandler.setUserPassword(pass);
									if (PasswordHandler.isUserPasswordValid()) PasswordValid = true;
								}
								if (PasswordValid) 
								{
									Document.decrypt();
									DocumentChanged = true;
									showMessage("The document was successfully decrypted", "Success");
								} 
								else 
                                    showErrorMessage("Invalid password", "Error");
							} 
							else if (Document.getEncryptionHandler() instanceof TElPDFPublicKeySecurityHandler) 
							{
								// requesting certificate from user
								if (RequestCertificate()) 
								{
                                    CertStorage.clear();
                                    CertStorage.add(Cert, true);
					                ((TElPDFPublicKeySecurityHandler)(Document.getEncryptionHandler())).setCertStorage(CertStorage);
                                    Document.decrypt();
									DocumentChanged = true;
									showMessage("The document was successfully decrypted", "Success");
								} 
								else 
									showErrorMessage("No certificate for decryption found", "Error");
							} 
							else 
							{
								showErrorMessage("The document is encrypted with unsupported security handler", "Error");
							}
						}
					}
                    // displaying signatures for signed documents
					if (Document.getSigned()) DisplaySignaturesInfo();
				} 
				finally 
				{
					// closing the document
					Document.close(DocumentChanged);
				}
			} 
			finally {	F.Free();	}
		} 
		catch(Exception ex) 
		{
			ex.printStackTrace();
			showErrorMessage("Error: " + ex.getMessage(), "Error");
			DocumentChanged = false;
		}
		// if the document was changed (e.g, decrypted), moving the temporary file to the place
		// of destination file
		if (DocumentChanged) 
		{
			String saveFileName = getPDFSaveFileName();
			if (saveFileName.length() > 0) {
				try {
					copyFile(TempPath, saveFileName, true);
					showMessage("The decrypted document was successfully saved", "Success");

				} catch (IOException e) {
					showErrorMessage(e.getMessage(), "Error");
					e.printStackTrace();
				}				
			}
		} 
		// deleting temporary file
        new File(TempPath).delete();
		dispose();
	}

	private String requestPassword(String msg) {
		JPasswordField jpf = new JPasswordField();
		int result = JOptionPane.showConfirmDialog(null, jpf, msg, JOptionPane.OK_CANCEL_OPTION);
		if (result == 0)
			return new String(jpf.getPassword());
		return "";
	}

	private boolean RequestCertificate() 
	{
		String fileName = getCertFileName();
		if (fileName.length() > 0)
		{
			try 
			{
				int R = 0;
				FileInputStream F = new FileInputStream(fileName);
				try 
				{
					String Pass = requestPassword("Password is needed to decrypt the certificate:");
					R = Cert.loadFromStreamPFX(F, Pass, 0);
				}
				finally { F.close();}
				if (R != 0) 
					showErrorMessage("Failed to load certificate, error " + R, "Error");
				else 
					return(true);
			} 
			catch(Exception ex) 
			{
				showErrorMessage("Failed to read certificate " + ex.getMessage(), "Error");
			}
		}
		return false;
	}

	private String getCertFileName() {
		JFileChooser fc = new JFileChooser();
		
		FileNameExtensionFilter filter = new FileNameExtensionFilter("PFX files", "pfx");
	    fc.setFileFilter(filter);	
	    
	    int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}

	private boolean DisplayEncryptionInfo() 
	{
		EncryptionPropsDialog dlg = new EncryptionPropsDialog();
		dlg.Init(Document);
		dlg.setModal(true);
		dlg.setVisible(true);
		return (dlg.isOK());
	}

	private void DisplaySignaturesInfo() 
	{
        SigPropsDialog dlg = new SigPropsDialog();
		dlg.Init(Document);
		dlg.setModal(true);
		dlg.setVisible(true);
	}
	
	protected String getPDFFileName() {
		JFileChooser fc = new JFileChooser();
		
		FileNameExtensionFilter filter = new FileNameExtensionFilter("PDF files", "pdf");
	    fc.setFileFilter(filter);	
	    
	    int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}

	protected String getPDFSaveFileName() {
		JFileChooser fc = new JFileChooser();
		
		FileNameExtensionFilter filter = new FileNameExtensionFilter("PDF files", "pdf");
	    fc.setFileFilter(filter);	
	    
	    int returnVal = fc.showSaveDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}
	
	void showErrorMessage(String msg, String cap){
		JOptionPane.showMessageDialog(null, msg, cap, JOptionPane.ERROR_MESSAGE);
	}
}
