

import java.awt.EventQueue;

import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPasswordField;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.filechooser.FileNameExtensionFilter;

import org.freepascal.rtl.TObject;

import SecureBlackbox.Base.*;
import SecureBlackbox.HTTPClient.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.Date;


public class Main extends JDialog {

	private static final long serialVersionUID = 1L;
	private JTextField edtCert;
	private JTextField edtIssuerCerts;
	private JTextField edtSignCert;
	private JTextField edtURL;
	
	private TElHTTPOCSPClient OCSPClient;
	private TElHTTPSClient HTTPSClient;
	private TElMemoryCertStorage CertStorage;
	private TElMemoryCertStorage IssuerCertStorage;
	private TElMemoryCertStorage SigningCertStorage;
	private JCheckBox cbSignRequest;
	

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					Main dialog = new Main();
					dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
					dialog.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the dialog.
	 */
	public Main() {
		setTitle("OCSP Client");
		setBounds(100, 100, 392, 330);
		getContentPane().setLayout(null);
		
		JLabel lblCertificateToCheck = new JLabel("Certificate to check");
		lblCertificateToCheck.setBounds(10, 11, 269, 14);
		getContentPane().add(lblCertificateToCheck);
		
		edtCert = new JTextField();
		edtCert.setBounds(10, 36, 269, 20);
		getContentPane().add(edtCert);
		edtCert.setColumns(10);
		
		JButton btnBrowseCert = new JButton("Browse");
		btnBrowseCert.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				edtCert.setText(getOpenCertFile());
			}
		});
		btnBrowseCert.setBounds(283, 35, 91, 23);
		getContentPane().add(btnBrowseCert);
		
		JLabel lblIssuerCertificates = new JLabel("Issuer certificate(s)");
		lblIssuerCertificates.setBounds(10, 67, 217, 14);
		getContentPane().add(lblIssuerCertificates);
		
		edtIssuerCerts = new JTextField();
		edtIssuerCerts.setColumns(10);
		edtIssuerCerts.setBounds(10, 93, 269, 20);
		getContentPane().add(edtIssuerCerts);
		
		JButton buttonBrowseIssCert = new JButton("Browse");
		buttonBrowseIssCert.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				edtIssuerCerts.setText(getOpenCertFile());
			}
		});
		buttonBrowseIssCert.setBounds(283, 92, 91, 23);
		getContentPane().add(buttonBrowseIssCert);
		
		cbSignRequest = new JCheckBox("Sign the request");
		cbSignRequest.setBounds(10, 120, 155, 23);
		getContentPane().add(cbSignRequest);
		
		JLabel lblCertificatesToSign = new JLabel("Certificate(s) to sign the request:");
		lblCertificatesToSign.setBounds(10, 150, 304, 14);
		getContentPane().add(lblCertificatesToSign);
		
		edtSignCert = new JTextField();
		edtSignCert.setColumns(10);
		edtSignCert.setBounds(10, 176, 269, 20);
		getContentPane().add(edtSignCert);
		
		JButton buttonBrowseSignCert = new JButton("Browse");
		buttonBrowseSignCert.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				edtSignCert.setText(getOpenCertFile());
			}
		});
		buttonBrowseSignCert.setBounds(283, 175, 91, 23);
		getContentPane().add(buttonBrowseSignCert);
		
		JLabel lblOcspServerUrl = new JLabel("OCSP server URL:");
		lblOcspServerUrl.setBounds(10, 207, 128, 14);
		getContentPane().add(lblOcspServerUrl);
		
		edtURL = new JTextField();
		edtURL.setBounds(10, 228, 269, 20);
		getContentPane().add(edtURL);
		edtURL.setColumns(10);
		
		JButton btnCheck = new JButton("Check");
		btnCheck.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				checkClick();
			}
		});
		btnCheck.setBounds(10, 269, 91, 23);
		getContentPane().add(btnCheck);

		init();
	}

	protected void checkClick() {
		String[] status = {"Success", "Malformed request", "Internal error", "Try later", "", "Signature required", "Unauthorized"};

		int res;
		short ServerResult = 0;
		boolean Reason = false;
		TElCertificateValidityInfo valres = new TElCertificateValidityInfo();
		int i;

		LoadCertificates(edtCert.getText(), CertStorage);
		LoadCertificates(edtIssuerCerts.getText(), IssuerCertStorage);
		OCSPClient.setIncludeSignature(cbSignRequest.isSelected());
		if (OCSPClient.getIncludeSignature())
		{
			LoadCertificates(edtSignCert.getText(), SigningCertStorage);
		}
		OCSPClient.setNonce(SBUtils.bytesOfString(new Date().toString())); // In real life use some cryptographically strong random data
		
		OCSPClient.setURL(edtURL.getText());
		TElOCSPServerResultWithReply repl = new TElOCSPServerResultWithReply();
		res = OCSPClient.performRequest(repl);
		if (res != 0)
		{
			switch ((int) res)
			{
				case SBOCSPCommon.SB_OCSP_ERROR_NO_PARAMETERS: 
					showErrorMessage("URL not specified");
					break;
				case SBOCSPCommon.SB_OCSP_ERROR_NO_REPLY: 
					showErrorMessage("Failed to retrieve a reply from OCSP server");
					break;
				case SBOCSPCommon.SB_OCSP_ERROR_WRONG_SIGNATURE: 
					showErrorMessage("Reply from OCSP server contains invalid or broken signature");
					break;
				case SBOCSPCommon.SB_OCSP_ERROR_NO_CERTIFICATES: 
					showErrorMessage("No certificates have been specified for checking");
					break;
				case SBOCSPCommon.SB_OCSP_ERROR_NO_ISSUER_CERTIFICATES: 
					showErrorMessage("No issuer certificates were found");
					break;
				default:
					showErrorMessage("Error " + res + " happened when trying to check certificate status");
					break;
			}
		}
		else
		{
			showMessage("The server replied with the following status: " + status[ServerResult]);
			if (ServerResult == SBOCSPCommon.oseSuccessful)
			{
				// First validate the server certificates
				if (OCSPClient.getReplyCertificates().getCount() == 0)
					showMessage("The server didn\'t include signing certificates to the reply");
				for (i = OCSPClient.getReplyCertificates().getCount() - 1; i >= 0; i--)
				{
					valres = OCSPClient.getReplyCertificates().validate(OCSPClient.getReplyCertificates().getCertificate(i), Reason, new Date());
					if (valres.Validity == TSBCertificateValidity.cvInvalid)
						break;
				}
				if (valres.Validity == TSBCertificateValidity.cvInvalid) 
					showMessage("One of certificates, used to sign the reply, is not valid");

				// Next, check certificate's status
				for (i = 0; i < CertStorage.getCount(); i++)
				{
					switch (OCSPClient.getCertStatus(i))
					{
						case SBOCSPCommon.csGood: 
							showMessage("Certificate " + i + " is ok");
							break;
						case SBOCSPCommon.csRevoked: 
							showMessage("Certificate " + i + " has been revoked on " + OCSPClient.getRevocationTime(i).toString());
							break;
						case SBOCSPCommon.csUnknown: 
							showErrorMessage("Certificate " + i + " is not known to OCSP server");
							break;
					}
				}
			}
		}
	}

	protected String getOpenCertFile() {
		JFileChooser fc = new JFileChooser();
		fc.setDialogTitle("Select certificate file");

		FileNameExtensionFilter filter = new FileNameExtensionFilter("( PEM | DER | PFX | SPC | PKCS#7 )-encoded certificate", "pem", "cer", "pfx", "spc", "p7b");
		fc.setFileFilter(filter);

	    int returnVal = fc.showOpenDialog(this);
		if (returnVal == JFileChooser.APPROVE_OPTION)
	        return fc.getSelectedFile().getPath();

	    return "";
	}

	private void init() {
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
		SBUtils.setLicenseKey("<put the key here>");
		
		OCSPClient = new TElHTTPOCSPClient();
		HTTPSClient = new TElHTTPSClient();
		CertStorage = new TElMemoryCertStorage();
		IssuerCertStorage = new TElMemoryCertStorage();
		SigningCertStorage = new TElMemoryCertStorage();
		
		HTTPSClient.setHTTPProxyPort(3128);
		HTTPSClient.setHTTPVersion(TSBHTTPVersion.hvHTTP10);
		HTTPSClient.setSendBufferSize(65535);
		HTTPSClient.setSocksAuthentication(0);
		HTTPSClient.setUseCompression(true);
		HTTPSClient.setWebTunnelPort(3128);
		
		OCSPClient.setCertStorage(CertStorage);
		OCSPClient.setHTTPClient(HTTPSClient);
		OCSPClient.setIncludeCertificates(true);
		OCSPClient.setIncludeSignature(false);
		OCSPClient.setIssuerCertStorage(IssuerCertStorage);
		OCSPClient.setSigningCertStorage(SigningCertStorage);
		
		OCSPClient.setOnCertificateNeeded(new TSBOCSPCertificateNeededEvent(OnCertificateNeeded));

	}
	
	TSBOCSPCertificateNeededEvent.Callback OnCertificateNeeded = new TSBOCSPCertificateNeededEvent.Callback() {
		
		public TElX509Certificate tsbocspCertificateNeededEventCallback(TObject sender) {
			// in real life you need to look for certificates with
			// subject name equal to OCSPClient.ServerName
			return null;
		}
	};
	
 	private String AskForPassword(String filename) {
		JPasswordField jpf = new JPasswordField();
		int result = JOptionPane.showConfirmDialog(null, jpf, "Please enter password " + filename, JOptionPane.OK_CANCEL_OPTION);
		if (result == 0)
			return new String(jpf.getPassword());
		return "";
	}

	private void showMessage(String msg) {
		JOptionPane.showMessageDialog(null, msg, getTitle(), JOptionPane.INFORMATION_MESSAGE);		
	}
 	
	private void LoadCertificates(String filename, TElCustomCertStorage storage)
	{
		int Format;
		TElX509Certificate Cert;
		TElFileStream Stream;

		storage.clear();
		Format = TElX509Certificate.detectCertFileFormat(filename);
		if (Format == SBX509.cfUnknown)
		{
			showMessage(filename + " doesn\'t contain valid certificate(s)");
			return;
		}
		Cert = new TElX509Certificate(null);
		try
		{
			Stream = new TElFileStream(filename, "r", true);
			switch (Format)
			{
				case SBX509.cfDER: 
					Cert.loadFromStream(Stream, (int) Stream.getLength());
					break;
				case SBX509.cfPEM: 
					if (Cert.loadFromStreamPEM(Stream, "", (int) Stream.getLength()) == SBPEM.PEM_DECODE_RESULT_INVALID_PASSPHRASE)
						SBPEM.raisePEMError(Cert.loadFromStreamPEM(Stream, AskForPassword(filename), (int) Stream.getLength()));
					break;
				case SBX509.cfPFX: 
					if (Cert.loadFromStreamPFX(Stream, "", (int) Stream.getLength()) == SBPKCS12.SB_PKCS12_ERROR_INVALID_PASSWORD)
						SBPKCS12.raisePKCS12Error(Cert.loadFromStreamPFX(Stream, AskForPassword(filename), (int) Stream.getLength()));
					break;
				case SBX509.cfSPC: 
					SBPKCS7.raisePKCS7Error(Cert.loadFromStreamSPC(Stream, (int) Stream.getLength()));
					break;
			}
			storage.add(Cert, true);
		}
		catch (Exception ex)
		{
			showErrorMessage("Failed to load the certificate(s) from " + filename + ": \n" + ex.getMessage());
		}
	} 	

	private void showErrorMessage(String msg) {
		JOptionPane.showMessageDialog(null, msg, getTitle(), JOptionPane.ERROR_MESSAGE);
	}

}
