

import java.awt.EventQueue;

import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JPasswordField;
import javax.swing.JButton;

import SecureBlackbox.Base.*;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;


public class Main extends JDialog {

	private static final long serialVersionUID = 1L;
	private JTextField tbInput;
	private JPasswordField tbPassword;
	private JTextField tbEncrypted;
	private JTextField tbDecrypted;

	private TElSymmetricCryptoFactory factory = new TElSymmetricCryptoFactory(); 

	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					Main dialog = new Main();
					dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
					dialog.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the dialog.
	 */
	public Main() {
		setResizable(false);
		setTitle("Symmetric Encryption Sample");
		setBounds(100, 100, 340, 290);
		getContentPane().setLayout(null);
		
		JLabel lblInputString = new JLabel("Input string:");
		lblInputString.setBounds(10, 11, 115, 14);
		getContentPane().add(lblInputString);
		
		tbInput = new JTextField();
		tbInput.setBounds(10, 28, 314, 20);
		getContentPane().add(tbInput);
		tbInput.setColumns(10);
		
		JLabel lblPassword = new JLabel("Password:");
		lblPassword.setBounds(10, 59, 131, 14);
		getContentPane().add(lblPassword);
		
		tbPassword = new JPasswordField();
		tbPassword.setBounds(10, 76, 314, 20);
		getContentPane().add(tbPassword);
		
		JLabel lblEncryptedOutput = new JLabel("Encrypted output:");
		lblEncryptedOutput.setBounds(10, 107, 154, 14);
		getContentPane().add(lblEncryptedOutput);
		
		tbEncrypted = new JTextField();
		tbEncrypted.setBounds(10, 125, 314, 20);
		getContentPane().add(tbEncrypted);
		tbEncrypted.setColumns(10);
		
		JLabel lblDecryptedString = new JLabel("Decrypted string:");
		lblDecryptedString.setBounds(10, 156, 165, 14);
		getContentPane().add(lblDecryptedString);
		
		tbDecrypted = new JTextField();
		tbDecrypted.setEnabled(false);
		tbDecrypted.setBounds(10, 173, 314, 20);
		getContentPane().add(tbDecrypted);
		tbDecrypted.setColumns(10);
		
		JButton btnEncrypt = new JButton("Encrypt");
		btnEncrypt.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				encryptClick();
			}
		});
		btnEncrypt.setBounds(66, 229, 91, 23);
		getContentPane().add(btnEncrypt);
		
		JButton btnDecrypt = new JButton("Decrypt");
		btnDecrypt.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				decryptClick();
			}
		});
		btnDecrypt.setBounds(167, 229, 91, 23);
		getContentPane().add(btnDecrypt);

		init();
	}

	private void init() {
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
		SBUtils.setLicenseKey("<put the key here>");
	}

	protected void decryptClick() {
		try
		{
			TElSymmetricCrypto crypto = factory.createInstance(SBConstants.SB_ALGORITHM_CNT_AES256, TSBSymmetricCryptoMode.cmDefault);
			crypto.setKeyMaterial(passwordToKeyMaterial(new String(tbPassword.getPassword())));

			ByteArrayInputStream bis = new ByteArrayInputStream(SBEncoding.base64DecodeArray(tbEncrypted.getText()));
			ByteArrayOutputStream bos = new ByteArrayOutputStream();

			crypto.decrypt(bis, bos, 0);

			byte[] outBuf = bos.toByteArray();
			tbDecrypted.setText(SBUtils.stringOfBytes(outBuf, 0, outBuf.length));
		}
		catch (Exception ex)
		{
			showErrorMessage("Decryption error: " + ex.getMessage(), "Error");
		}
	}

	protected void encryptClick() {
		try
		{
			TElSymmetricCrypto crypto = factory.createInstance(SBConstants.SB_ALGORITHM_CNT_AES256, TSBSymmetricCryptoMode.cmDefault);
			crypto.setKeyMaterial(passwordToKeyMaterial(new String(tbPassword.getPassword())));

			ByteArrayInputStream bis = new ByteArrayInputStream(SBUtils.bytesOfString(tbInput.getText()));
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			
			crypto.encrypt(bis, bos);

			// convert binary output to Base64 to make it readable
			tbEncrypted.setText(SBEncoding.base64EncodeArray(bos.toByteArray(), false));
		}
		catch (Exception ex)
		{
			showErrorMessage("Encryption error: " + ex.getMessage(), "Error");
		}
	}

	static void showErrorMessage(String msg, String cap) {
		JOptionPane.showMessageDialog(null, msg, cap, JOptionPane.ERROR_MESSAGE);
	}	
	
	// This function converts a password into an encryption key using SHA-256 hash
	// function. Initialization vector is filled with zeros.
	// NEVER do this in real life since this makes security void.
	// Instead you should convert a password to encryption key using some
	// password derivation function (see PKCS#5 for details).
	private TElSymmetricKeyMaterial passwordToKeyMaterial(String pass) {
		TElHashFunction hash = new TElHashFunction(SBConstants.SB_ALGORITHM_DGST_SHA256, (TElCustomCryptoProvider)null);
		hash.update(SBUtils.bytesOfString(pass));
		byte[] digest = hash.finish();

		TElSymmetricKeyMaterial res = new TElSymmetricKeyMaterial(null);
		res.setKey(digest); // set 256-bit key
		res.setIV(new byte[16]); // set 128-bit initialization vector

		return res;
	}	
}
