

import org.freepascal.rtl.TObject;

import SecureBlackbox.Base.*;
import SecureBlackbox.ZIP.*;


public class Main {
		public final static String helpMessage = 
            "Usage: consolezip.exe [flags] [operation] input1, input2, .... [-o output]\n\n" +
            "Where possible operations are: \n" +
            "-c : compress input files (or directories, wildcards allowed), and save archive to output\n" +
            "-d : decompress input archives (no wildcards!), and write files to output directory\n" +
            "Where possible flags are:\n" +
            "-e [encryption mode]: encrypt with [encryption mode], where possible modes are: weak, winzip [aes128|aes192|aes256], strong [aes128|aes192|aes256|3des] [dir]\n" +
            "-m [deflate|bzip2|store] [1..9]: set compression algorithm and compression level\n" +
            "-p password: set password for encryption/decryption";
        public final String compressionLevels = "123456789";

        public TElStringList inputFiles;
        public String outputPath;
        public boolean compressMode, decompressMode, passwordAsked;
        public int FLastProgress;
        public TElZipWriter zipWriter;

        public Main()
        {
            #error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
        	SBUtils.setLicenseKey("<put the key here>");

            inputFiles = new TElStringList();
            compressMode = false;
            decompressMode = false;
            passwordAsked = false;
            outputPath = "";

            zipWriter = new TElZipWriter(null);
            
            initEvents();
        }

        private void initEvents() {
            zipWriter.setOnCompressionStart(new TSBZipCompressionStartEvent(onCompressionStart));
            zipWriter.setOnCompressionFinished(new TSBZipCompressionFinishedEvent(onCompressionFinished));
            zipWriter.setOnProgress(new TSBZipProgressEvent(onProgress));
            zipWriter.setOnExtractionStart(new TSBZipExtractionStartEvent(onExtractionStart));
            zipWriter.setOnExtractionFinished(new TSBZipExtractionFinishedEvent(onExtractionFinished));
            zipWriter.setOnPasswordNeeded(new TSBZipPasswordNeededEvent(onPasswordNeeded));
			zipWriter.setOnUserActionNeeded(new TSBZipUserActionNeededEvent(onUserActionNeeded));
		}

        TSBZipUserActionNeededEvent.Callback onUserActionNeeded = new TSBZipUserActionNeededEvent.Callback() {
			
			public void tsbZipUserActionNeededEventCallback(TObject sender, int forEvent,
					String description, TObject param, TSBInteger action) {
					int userAction = 0;
					// user actions handling 
		            if (forEvent == SBArcZip.ZIP_EVENT_CRC_MISMATCH)
		            {
		                System.out.print(description);
		                userAction = SBArcZip.ZIP_ACTION_IGNORE;
		            }
		            else if (forEvent == SBArcZip.ZIP_EVENT_FILE_ALREADY_EXISTS)
		                userAction = SBArcZip.ZIP_ACTION_IGNORE;
		            else if (forEvent == SBArcZip.ZIP_EVENT_DIR_ALREADY_EXISTS)
		                userAction = SBArcZip.ZIP_ACTION_IGNORE;
		            else
		                userAction = SBArcZip.ZIP_ACTION_ABORT;
		       
		            action.value = userAction;
			}
		};
        
        TSBZipPasswordNeededEvent.Callback onPasswordNeeded = new TSBZipPasswordNeededEvent.Callback() {
			
			public void tsbZipPasswordNeededEventCallback(TObject sender,
					TObject param, TSBString passphrase, TSBBoolean accept) {
	            System.out.print("Password is needed: ");
	            passphrase.value = ReadPassword();
				accept.value = true;
			}
		};
        
        TSBZipExtractionFinishedEvent.Callback onExtractionFinished = new TSBZipExtractionFinishedEvent.Callback() {
			
			public void tsbZipExtractionFinishedEventCallback(TObject sender,
					TElZipArchiveDirectoryEntry fLastProgress) {
	            for (int i = fLastProgress.getEntriesCount() + 1; i <= 10; i++)
	                System.out.print('.');

	            System.out.println("] OK");				
			}
		};
        
        TSBZipExtractionStartEvent.Callback onExtractionStart = new TSBZipExtractionStartEvent.Callback() {
			
			public void tsbZipExtractionStartEventCallback(TObject sender,
					TElZipArchiveDirectoryEntry entry, TSBBoolean accept) {
	            if (entry.getEncrypted() && (!passwordAsked))
	            {
	            	String pass = null;
	            	
	            	do {
	            		System.out.print("Password is needed: ");
	            		pass = ReadPassword();
	            	} while (pass == null);
	            	
	                zipWriter.setPassword(pass);
	                passwordAsked = true;
	            }

	            System.out.print("Extracting " + entry.getPath() + " (" + SBZipUtils.zipCompressionMethodName(entry.getCompressionMethod()) + ") [");
	            FLastProgress = 0;

				accept.value = true;
			}
		};
        
        TSBZipProgressEvent.Callback onProgress = new TSBZipProgressEvent.Callback() {
			
			public void tsbZipProgressEventCallback(TObject sender, long processed,
					long total, long overallProcessed, long overallTotal, TSBBoolean cancel) {
		            int i, newProgress;

		            newProgress = (int)((processed * 10) / total);
		            if (newProgress > FLastProgress)
		            {
		                for (i = FLastProgress + 1; i <= newProgress; i++)
		                    System.out.print('.');
		                FLastProgress = newProgress;
		            }
		            cancel.value = false;
			}
		};
        
        TSBZipCompressionFinishedEvent.Callback onCompressionFinished = new TSBZipCompressionFinishedEvent.Callback() {
			
			public void tsbZipCompressionFinishedEventCallback(TObject sender,
					TElZipArchiveDirectoryEntry FLastProgress) {
	            for (int i = FLastProgress.getEntriesCount() + 1; i <= 10; i++)
	                System.out.print('.');

	            System.out.println("] OK");
				
			}
		};
		
        TSBZipCompressionStartEvent.Callback onCompressionStart = new TSBZipCompressionStartEvent.Callback() {
			
			public void tsbZipCompressionStartEventCallback(TObject sender,
					TElZipArchiveDirectoryEntry entry, TSBBoolean allow) {
	            System.out.print("Compressing " + entry.getPath() + " (" + SBZipUtils.zipCompressionMethodName(entry.getCompressionMethod()) + ") [");
	            FLastProgress = 0;
				allow.value = true;
			}
		};
        
		public String GetEncryptionInfo()
        {
            String result = "";
            if (!zipWriter.getEncrypt())
                result = "none";
            else if (!(zipWriter.getStrongEncryption() || zipWriter.getWinZipEncryption()))
                result = "weak";
            else if (zipWriter.getWinZipEncryption())
                result = "WinZip, AES-" + zipWriter.getWinZipAesKeySize();
            else if (zipWriter.getStrongEncryption())
            {  
                result = "Strong encryption, " + SBZipUtils.zipEncryptionAlgorithmName(zipWriter.getStrongEncryptionInfo().getEncryptionAlgorithm());
                if (zipWriter.getDirectoryEncrypted())
                    result = result + ", encrypted directory";
            }
            return result;
        }

        public String UnQuote(String st)
        {
            if (st.length() < 1)
                return st;

            if ((st.charAt(0) == '"') && (st.charAt(st.length() - 1) == '"'))
                return st.substring(1, st.length() - 2);
            else
                return st;
        }

        public String ReadPassword()
        {
        	return System.console().readLine();
        }


        void ParseInputParameters(String[] ParamStr) throws Exception
        {
            int ParamNum = 0, ParamCount = ParamStr.length - 1;
            String Param;

            // parsing input parameters 

            while (ParamNum <= ParamCount)
            {
                Param = ParamStr[ParamNum];
                ParamNum++;

                // compression or decompression - input files list
                if ((Param.compareTo("-c") == 0) || (Param.compareTo("-d") == 0))
                {
    	            if (compressMode || decompressMode)
                        throw new Exception("Invalid usage of parameter: " + Param);

                    if (Param.compareTo("-c") == 0)
                        compressMode = true;
                    else
                        decompressMode = true;

                    while ((ParamNum <= ParamCount) && (ParamStr[ParamNum].charAt(0) != '-'))
                    {
                        inputFiles.add(UnQuote(ParamStr[ParamNum]));
                        ParamNum++;
                    }
                }
                // encryption options
                else if (Param.compareTo("-e") == 0)
                {
                    zipWriter.setEncrypt(true);

                    while ((ParamNum <= ParamCount) && (ParamStr[ParamNum].charAt(0) != '-'))
                    {
                        Param = ParamStr[ParamNum];
                        ParamNum++;
        
                        if (Param.compareTo("weak") == 0)
                        {
                            zipWriter.setStrongEncryption(false);
                            zipWriter.setWinZipEncryption(false);
                        }
                        else if (Param.compareTo("strong") == 0)
                        {
                            zipWriter.setStrongEncryption(true);
                            zipWriter.setWinZipEncryption(false);
                        }
                        else if (Param.compareTo("winzip") == 0)
                        {
                            zipWriter.setStrongEncryption(false);
                            zipWriter.setWinZipEncryption(true);
                        }
                        else if (Param.compareTo("aes128") == 0)
                        {
                            zipWriter.setWinZipAesKeySize(128);
                            zipWriter.getStrongEncryptionInfo().setEncryptionAlgorithm(SBZipUtils.ZIP_ENCRYPTION_AES128);
                        }
                        else if (Param.compareTo("aes192") == 0)
                        {
                            zipWriter.setWinZipAesKeySize(192);
                            zipWriter.getStrongEncryptionInfo().setEncryptionAlgorithm(SBZipUtils.ZIP_ENCRYPTION_AES192);
                        }
                        else if (Param.compareTo("aes256") == 0)
                        {
                            zipWriter.setWinZipAesKeySize(256);
                            zipWriter.getStrongEncryptionInfo().setEncryptionAlgorithm(SBZipUtils.ZIP_ENCRYPTION_AES256);
                        }
                        else if (Param.compareTo("3des") == 0)
                        {
                            zipWriter.getStrongEncryptionInfo().setEncryptionAlgorithm(SBZipUtils.ZIP_ENCRYPTION_3DES168);
                        }
                        else if (Param.compareTo("dir") == 0)
                        {
                            zipWriter.setDirectoryCompressed(true);
                            zipWriter.setDirectoryEncrypted(true);
                        }
                        else
                            throw new Exception("Invalid usage of parameter: " + Param);
                    }
                }
                else if (Param.compareTo("-m") == 0)
                {
                    while ((ParamNum <= ParamCount) && (ParamStr[ParamNum].charAt(0) != '-'))
                    {
                        Param = ParamStr[ParamNum];
                        ParamNum++;

                        if (Param.compareTo("deflate") == 0)
                            zipWriter.setCompressionAlgorithm(SBZipUtils.ZIP_COMPRESSION_DEFLATE);
                        else if (Param.compareTo("bzip2") == 0)
                            zipWriter.setCompressionAlgorithm(SBZipUtils.ZIP_COMPRESSION_BZIP2);
                        else if (Param.compareTo("stored") == 0)
                            zipWriter.setCompressionAlgorithm(SBZipUtils.ZIP_COMPRESSION_STORED);
                        else if (compressionLevels.indexOf(Param) > 0)
                            zipWriter.setCompressionLevel(compressionLevels.indexOf(Param));
                        else
                            throw new Exception("Invalid usage of parameter: " + Param);
                    }
                }
                else if (Param.compareTo("-p") == 0)
                {
                    if (ParamNum > ParamCount)
                        throw new Exception("Invalid usage of parameter: " + Param);

                    zipWriter.setPassword(UnQuote(ParamStr[ParamNum]));
                    ParamNum++;
                }
                else if (Param.compareTo("-o") == 0)
                {
                    if (ParamNum > ParamCount)
                        throw new Exception("Invalid usage of parameter: " + Param);

                    outputPath = UnQuote(ParamStr[ParamNum]);
                    ParamNum++;
                }
                else if (Param.compareTo("-h") == 0)
                {
                    System.out.println(helpMessage);
                }
                else
                    throw new Exception("Invalid parameter: " + Param);
            }
        }

    	public static void main(String[] args) {
            int i;
            Main program = new Main();

            System.out.println("EldoS ZipBlackbox console demo.");

            try
            {
                try
                {
                    program.ParseInputParameters(args);
                }
                catch (Exception E)
                {
                    System.out.println("Parameters parsing error: " + E.getMessage());
                    System.out.println(helpMessage);
                    return;
                }

                // creating archive

                if (program.compressMode)
                {
                    if (program.zipWriter.getEncrypt() && (program.zipWriter.getPassword().length() == 0))
                    {
                        System.out.print("Enter password to encrypt archive:");
                        program.zipWriter.setPassword(program.ReadPassword());
                        if (program.zipWriter.getPassword().length() == 0)
                        {
                            System.out.println("Empty password: encryption is disabled");
                            program.zipWriter.setEncrypt(false);
                        }
                    }

                    if (program.outputPath.length() == 0)
                        program.outputPath = "archive.zip";

                    System.out.println("Creating archive " + program.outputPath);
                    System.out.println("Compression algorithm: " + SBZipUtils.zipCompressionMethodName(program.zipWriter.getCompressionAlgorithm()));
                    System.out.println("Encryption: " + program.GetEncryptionInfo());

                    try
                    {
                        program.zipWriter.createArchive();

                        for (i = 0; i < program.inputFiles.size(); i++)
                        {
                            program.zipWriter.add((String) program.inputFiles.get(i));
                            program.zipWriter.compress(program.outputPath);
                        }
                    }
                    catch (Exception E)
                    {
                        System.out.println("Compression error : " + E.getMessage());
                        return;
                    }
                }
                else if (program.decompressMode)
                {
                    if (program.outputPath.length() == 0)
                        program.outputPath = ".";

                    try
                    {
                        for (i = 0; i < program.inputFiles.size(); i++)
                        {
                            program.zipWriter.open((String) program.inputFiles.get(i), true);
                            program.zipWriter.extract("*.*", program.outputPath);
                            program.zipWriter.close();
                        }
                    }
                    catch (Exception E)
                    {
                        System.out.println("Decompression error : " + E.getMessage());
                        return;
                    }
                }
                else
                    System.out.println("No operation specified. Use consolezip.exe -h to see the list of available options.");
            }
            finally
            {
                program = null;
            }
        }
}
